#!/usr/bin/env python

from dm.daq_web_service.api.experimentDaqApi import ExperimentDaqApi
from dm.common.exceptions.invalidRequest import InvalidRequest
from daqWebServiceSessionCli import DaqWebServiceSessionCli

class StartDaqCli(DaqWebServiceSessionCli):

    HOURS_PER_DAY = 24

    def __init__(self):
        DaqWebServiceSessionCli.__init__(self, validArgCount=self.ANY_NUMBER_OF_POSITIONAL_ARGS)
        self.addOption('', '--experiment', dest='experimentName', help='Experiment name.')
        self.addOption('', '--data-directory', dest='dataDirectory', help='Experiment data directory. If specified string does not already contain file server URL, value of the %s environment variable will be prepended to it.' % self.DM_FILE_SERVER_URL_ENV_VAR)
        self.addOption('', '--target-directory', dest='targetDirectory', help='Target directory relative to experiment root path.')
        self.addOption('', '--run-time', dest='runTime', help='Maximum run time; it must be specified in hours (h) or days (d). Examples: "8h", "14d".')
        self.addOption('', '--upload-data-directory-on-exit', dest='uploadDataDirectoryOnExit', help='Data directory that will be uploaded automatically after DAQ is stopped.')
        self.addOption('', '--upload-target-directory-on-exit', dest='uploadTargetDirectoryOnExit', help='Target directory relative to experiment root path for automatic upload after DAQ is stopped. Requires upload data directory to be specified.')
        self.addOption('', '--process-hidden', dest='processHidden', action='store_true', default=False, help='Process hidden source files.')

    def checkArgs(self):
        if self.options.experimentName is None:
            raise InvalidRequest('Experiment name must be provided.')
        if self.options.dataDirectory is None:
            raise InvalidRequest('Experiment data directory must be provided.')

    def updateDaqInfoFromOptions(self, daqInfo):
        if self.options.processHidden:
            daqInfo['processHiddenFiles'] = True
        if self.options.runTime:
            runTime = self.options.runTime
            if runTime.endswith('h'):
                daqInfo['maxRunTimeInHours'] = int(runTime[0:-1])
            elif runTime.endswith('d'):
                daqInfo['maxRunTimeInHours'] = int(runTime[0:-1])*self.HOURS_PER_DAY
            else:
                raise InvalidRequest('Maximum run time must contain valid unit specifier: "h" for hours or "d" for days.')
        if self.options.targetDirectory:
            daqInfo['targetDirectory'] = self.options.targetDirectory 
        if self.options.uploadDataDirectoryOnExit:
            daqInfo['uploadDataDirectoryOnExit'] = self.options.uploadDataDirectoryOnExit
        if self.options.uploadTargetDirectoryOnExit:
            if not self.options.uploadDataDirectoryOnExit:
                raise InvalidRequest('Upload target directory on exit requires that upload data directory is specified as well.')
            daqInfo['uploadTargetDirectoryOnExit'] = self.options.uploadTargetDirectoryOnExit

    def runCommand(self):
        self.parseArgs(usage="""
    dm-start-daq --experiment=EXPERIMENTNAME --data-directory=DATADIRECTORY
        [--run-time=RUNTIME]
        [--target-directory=TARGETDIRECTORY]
        [--upload-data-directory-on-exit=UPLOADDATADIRECTORYONEXIT]
        [--upload-target-directory-on-exit=UPLOADTARGETDIRECTORYONEXIT]
        [--process-hidden]
        [key1:value1, key2:value2, ...]

Description:
    Starts DAQ for a given experiment. Specified data directory will be 
    monitored for data files. All provided key/value pairs will be passed to 
    file processing plugins.
        """)
        self.checkArgs()
        api = ExperimentDaqApi(self.getLoginUsername(), self.getLoginPassword(), self.getServiceHost(), self.getServicePort(), self.getServiceProtocol())
        daqInfo = self.splitArgsIntoDict()
        self.updateDaqInfoFromOptions(daqInfo)
        daqInfo = api.startDaq(self.getExperimentName(), self.getDataDirectory(), daqInfo=daqInfo)
        print daqInfo.getDisplayString(self.getDisplayKeys(), self.getDisplayFormat())

#######################################################################
# Run command.
if __name__ == '__main__':
    cli = StartDaqCli()
    cli.run()

